package com.ycwl.basic.controller.viid;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.ycwl.basic.annotation.IgnoreToken;
import com.ycwl.basic.mapper.DeviceMapper;
import com.ycwl.basic.mapper.FaceSampleMapper;
import com.ycwl.basic.mapper.SourceMapper;
import com.ycwl.basic.model.pc.device.entity.DeviceEntity;
import com.ycwl.basic.model.pc.faceSample.entity.FaceSampleEntity;
import com.ycwl.basic.model.pc.source.entity.SourceEntity;
import com.ycwl.basic.model.viid.entity.DeviceIdObject;
import com.ycwl.basic.model.viid.entity.FaceListObject;
import com.ycwl.basic.model.viid.entity.FaceObject;
import com.ycwl.basic.model.viid.entity.FacePositionObject;
import com.ycwl.basic.model.viid.entity.ResponseStatusObject;
import com.ycwl.basic.model.viid.entity.SubImageInfoObject;
import com.ycwl.basic.model.viid.entity.SubImageList;
import com.ycwl.basic.model.viid.entity.SystemTimeObject;
import com.ycwl.basic.model.viid.req.FaceUploadReq;
import com.ycwl.basic.model.viid.req.KeepaliveReq;
import com.ycwl.basic.model.viid.req.RegisterReq;
import com.ycwl.basic.model.viid.req.UnRegisterReq;
import com.ycwl.basic.model.viid.resp.SystemTimeResp;
import com.ycwl.basic.model.viid.resp.VIIDBaseResp;
import com.ycwl.basic.service.task.TaskFaceService;
import com.ycwl.basic.storage.StorageFactory;
import com.ycwl.basic.storage.adapters.IStorageAdapter;
import com.ycwl.basic.utils.ImageUtils;
import com.ycwl.basic.utils.IpUtils;
import com.ycwl.basic.utils.SnowFlakeUtil;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;
import java.util.UUID;

@IgnoreToken
@RestController
@Api(tags = "摄像头对接接口")
@RequestMapping("/VIID")
@Slf4j
public class ViidController {
    @Autowired
    private DeviceMapper deviceMapper;
    private static final String serverId = "00000000000000000001";
    @Autowired
    private SourceMapper sourceMapper;

    // region 注册注销基础接口
    /**
     * 注册接口
     *
     * @param req     注册的信息
     * @param request 请求
     * @return 返回
     */
    @RequestMapping(value = "/System/Register", method = RequestMethod.POST)
    public VIIDBaseResp register(@RequestBody RegisterReq req, HttpServletRequest request) {
        DeviceIdObject deviceIdObject = req.getRegisterObject();
        log.info("注册的设备信息：{}", deviceIdObject);
        // 保存设备注册时间
        DeviceEntity device = deviceMapper.getByDeviceNo(deviceIdObject.getDeviceId());
        if (device == null) {
            device = new DeviceEntity();
            device.setName("未配置设备");
            device.setNo(deviceIdObject.getDeviceId());
            device.setOnline(1);
        }
        device.setKeepaliveAt(new Date());
        device.setIpAddr(IpUtils.getIpAddr(request));
        if (device.getId() != null) {
            deviceMapper.updateEntity(device);
        } else {
            device.setId(SnowFlakeUtil.getLongId());
            deviceMapper.addEntity(device);
        }
        return new VIIDBaseResp(
                new ResponseStatusObject(serverId, "/VIID/System/Register", "0", "注册成功", sdfTime.format(new Date()))
        );
    }

    /**
     * 保活接口
     *
     * @param req     保活的设备信息
     * @param request 请求
     * @return 返回
     */
    @RequestMapping(value = "/System/Keepalive", method = RequestMethod.POST)
    public VIIDBaseResp keepalive(@RequestBody KeepaliveReq req, HttpServletRequest request) {
        DeviceIdObject keepaliveObject = req.getKeepaliveObject();
        log.info("对方发送的心跳的信息：{}", keepaliveObject);

        String deviceId = keepaliveObject.getDeviceId();
        DeviceEntity device = deviceMapper.getByDeviceNo(deviceId);

        // 判断设备状态
        if (device == null) {
            // 不存在设备就注册
            device = new DeviceEntity();
            device.setName("未配置设备");
            device.setNo(deviceId);
            device.setOnline(1);
            device.setKeepaliveAt(new Date());
            device.setIpAddr(IpUtils.getIpAddr(request));
            device.setId(SnowFlakeUtil.getLongId());
            deviceMapper.addEntity(device);
        } else {
            device.setOnline(1);
            device.setKeepaliveAt(new Date());
            deviceMapper.updateEntity(device);
        }
        log.info("已经解析过的心跳信息：{}", keepaliveObject);

        return new VIIDBaseResp(
                new ResponseStatusObject(deviceId, "/VIID/System/UnRegister", "0", "注销成功", sdfTime.format(new Date()))
        );
    }

    /**
     * 注销设备
     *
     * @param req     参数
     * @return 返回
     */
    @RequestMapping(value = "/System/UnRegister", method = RequestMethod.POST)
    public VIIDBaseResp unRegister(@RequestBody UnRegisterReq req) {
        // 获取设备id
        DeviceIdObject unRegisterObject = req.getUnRegisterObject();
        String deviceId = unRegisterObject.getDeviceId();
        log.info("获取的注销的请求参数：{}", unRegisterObject);

        // 首先查询该设备是否存在
        DeviceEntity device = deviceMapper.getByDeviceNo(deviceId);
        // 判断
        if (device != null) {
            device.setOnline(0);
            device.setKeepaliveAt(new Date());
            int update = deviceMapper.updateEntity(device);
        }
        return new VIIDBaseResp(
                new ResponseStatusObject(deviceId, "/VIID/System/UnRegister", "0", "注销成功", sdfTime.format(new Date()))
        );
    }

    /**
     * 校时接口
     *
     * @return 返回
     */
    @RequestMapping(value = "/System/Time", method = RequestMethod.GET)
    public SystemTimeResp time() {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        return new SystemTimeResp(
                new SystemTimeObject(serverId, "2", sdf.format(new Date()), TimeZone.getTimeZone("Asia/Shanghai").toString())
        );
    }

    // endregion

    @Autowired
    private FaceSampleMapper faceSampleMapper;

    @Autowired
    private TaskFaceService taskFaceService;
    private final SimpleDateFormat sdfTime = new SimpleDateFormat("yyyyMMddHHmmss");


    /**
     * 批量新增人脸
     */
    @RequestMapping(value = "/Faces", method = RequestMethod.POST)
    public VIIDBaseResp faces(@RequestBody FaceUploadReq req) {
        log.info("收到的人脸上报信息：{}",req);
        FaceListObject faceListObject = req.getFaceListObject();
        List<FaceObject> faceObject = faceListObject.getFaceObject();
        String faceId = null;
        // 遍历人脸列表
        for (FaceObject face : faceObject) {
            // 设置FaceId
            faceId = face.getFaceID();
            Long newFaceSampleId = SnowFlakeUtil.getLongId();
            // 获取图片信息
            SubImageList subImageList = face.getSubImageList();
            // 判断人脸对象中的列表是否为空
            String deviceID = face.getDeviceID();
            DeviceEntity device = deviceMapper.getByDeviceNo(deviceID);
            if (device == null) {
                continue;
            }
            Date shotTime = new Date();
            if (StringUtils.isNotBlank(face.getShotTime())) {
                try {
                    shotTime = sdfTime.parse(face.getShotTime());
                } catch (ParseException e) {
                    throw new RuntimeException(e);
                }
            }
            Long scenicId = device.getScenicId();
            if (scenicId == null) {
                continue;
            }
            FacePositionObject facePosition = new FacePositionObject();
            facePosition.setLtY(face.getLeftTopY());
            facePosition.setLtX(face.getLeftTopX());
            facePosition.setRbY(face.getRightBtmY());
            facePosition.setRbX(face.getRightBtmX());
            if (ObjectUtil.isNotEmpty(subImageList) && CollUtil.isNotEmpty(subImageList.getSubImageInfoObject())) {
                // 遍历每个图片对象
                for (SubImageInfoObject subImage : subImageList.getSubImageInfoObject()) {
                    // base64转换成MultipartFIle
                    MultipartFile file = ImageUtils.base64ToMultipartFile(subImage.getData());
                    String ext = subImage.getFileFormat();
                    if (ext.equalsIgnoreCase("jpeg")) {
                        ext = "jpg";
                    }
                    IStorageAdapter adapter = StorageFactory.use("faces");
                    // Type=11 人脸
                    if (subImage.getType().equals("11")) {
                        // 上传oss
                        FaceSampleEntity faceSample = new FaceSampleEntity();
                        faceSample.setId(newFaceSampleId);
                        faceSample.setScenicId(scenicId);
                        faceSample.setDeviceId(device.getId());
                        faceSample.setStatus(0);
                        faceSample.setCreateAt(shotTime);
                        String url = adapter.uploadFile(file, "user-face", UUID.randomUUID() + "." + ext);
                        faceSample.setFaceUrl(url);
                        faceSampleMapper.add(faceSample);
                        log.info("人脸信息入库成功！");
                        taskFaceService.addFaceSample(faceSample.getId());
                    }
                    // Type=14 场景图
                    else if (subImage.getType().equals("14")) {
                        facePosition.setImgHeight(subImage.getHeight());
                        facePosition.setImgWidth(subImage.getWidth());
                        SourceEntity source = new SourceEntity();
                        source.setId(SnowFlakeUtil.getLongId());
                        source.setDeviceId(device.getId());
                        source.setScenicId(device.getScenicId());
                        source.setFaceSampleId(newFaceSampleId);
                        source.setCreateTime(shotTime);
                        source.setType(2);
                        // 上传oss
                        String url = adapter.uploadFile(file, "user-photo", UUID.randomUUID() + "." + ext);
                        source.setUrl(url);
                        source.setPosJson(JSON.toJSONString(facePosition));
                        sourceMapper.add(source);
                    }
                }
            }
            log.info("设备ID：{}", deviceID);
        }

        return new VIIDBaseResp(
                new ResponseStatusObject(faceId, "/VIID/Faces", "0", "OK", sdfTime.format(new Date()))
        );
    }

}
