package com.ycwl.basic.service.impl.pc;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.ycwl.basic.mapper.TemplateMapper;
import com.ycwl.basic.model.pc.template.entity.TemplateConfigEntity;
import com.ycwl.basic.model.pc.template.entity.TemplateEntity;
import com.ycwl.basic.model.pc.template.req.TemplateReqQuery;
import com.ycwl.basic.model.pc.template.resp.TemplateRespVO;
import com.ycwl.basic.service.pc.TemplateService;
import com.ycwl.basic.template.repository.TemplateRepository;
import com.ycwl.basic.utils.ApiResponse;
import com.ycwl.basic.utils.SnowFlakeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Author：longbinbin
 * @Date：2024/12/3 16:20
 */
@Service
public class TemplateServiceImpl implements TemplateService {

    @Autowired
    private TemplateMapper templateMapper;
    @Autowired
    private TemplateRepository templateRepository;

    @Override
    public ApiResponse<PageInfo<TemplateRespVO>> pageQuery(TemplateReqQuery templateReqQuery) {
        PageHelper.startPage(templateReqQuery.getPageNum(), templateReqQuery.getPageSize());
        List<TemplateRespVO> list = templateMapper.list(templateReqQuery);
        PageInfo<TemplateRespVO> pageInfo = new PageInfo<>(list);
        return ApiResponse.success(pageInfo);
    }

    @Override
    public ApiResponse<List<TemplateRespVO>> list(TemplateReqQuery templateReqQuery) {
        return ApiResponse.success(templateMapper.list(templateReqQuery));
    }

    @Override
    public ApiResponse<TemplateRespVO> getById(Long id) {
        return ApiResponse.success(templateRepository.getTemplate(id));
    }

    @Override
    public ApiResponse<Boolean> add(TemplateEntity template) {
        template.setId(SnowFlakeUtil.getLongId());
        int i = templateMapper.add(template);
        if (template.getChildren() != null) {
            template.getChildren().forEach(item -> {
                item.setId(SnowFlakeUtil.getLongId());
                item.setPid(template.getId());
                item.setScenicId(template.getScenicId());
                item.setStatus(1);
                templateMapper.add(item);
            });
        }
        if (i > 0) {
            return ApiResponse.success(true);
        }else {
            return ApiResponse.fail("添加模版失败");
        }
    }

    @Override
    public ApiResponse<Integer> deleteById(Long id) {
        int i = templateMapper.deleteById(id);
        if (i > 0) {
            templateMapper.deleteByPid(id);
            templateRepository.clearTemplateCache(id);
            return ApiResponse.success(i);
        }else {
            return ApiResponse.fail("删除模版失败");
        }
    }

    @Override
    public ApiResponse<Boolean> update(TemplateEntity template) {
        int i = templateMapper.update(template);
        if (template.getChildren() != null) {
            templateMapper.deleteByPid(template.getId());
            template.getChildren().forEach(item -> {
                item.setId(SnowFlakeUtil.getLongId());
                item.setPid(template.getId());
                item.setScenicId(template.getScenicId());
                item.setStatus(1);
                templateMapper.add(item);
            });
            templateRepository.clearTemplateCache(template.getId());
        }
        if (i > 0) {
            return ApiResponse.success(true);
        }else {
            return ApiResponse.fail("更新模版失败");
        }
    }

    @Override
    public ApiResponse<Boolean> updateStatus(Long id) {
        int i = templateMapper.updateStatus(id);
        if (i > 0) {
            return ApiResponse.success(true);
        }else {
            return ApiResponse.fail("更新模版状态失败");
        }
    }

    @Override
    public TemplateConfigEntity getConfig(Long templateId) {
        return templateRepository.getTemplateConfig(templateId);
    }

    @Override
    public void saveConfig(Long configId, TemplateConfigEntity config) {
        config.setId(configId);
        templateRepository.clearTemplateCache(config.getTemplateId());
        templateMapper.updateConfigById(config);
    }
}
