package com.ycwl.basic.service.impl.task;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.facebody.model.v20191230.AddFaceEntityRequest;
import com.aliyuncs.facebody.model.v20191230.AddFaceRequest;
import com.aliyuncs.facebody.model.v20191230.AddFaceResponse;
import com.aliyuncs.facebody.model.v20191230.CreateFaceDbRequest;
import com.aliyuncs.facebody.model.v20191230.DeleteFaceDbRequest;
import com.aliyuncs.facebody.model.v20191230.DeleteFaceEntityRequest;
import com.aliyuncs.facebody.model.v20191230.ListFaceDbsRequest;
import com.aliyuncs.facebody.model.v20191230.ListFaceDbsResponse;
import com.aliyuncs.facebody.model.v20191230.ListFaceEntitiesRequest;
import com.aliyuncs.facebody.model.v20191230.ListFaceEntitiesResponse;
import com.aliyuncs.facebody.model.v20191230.SearchFaceRequest;
import com.aliyuncs.facebody.model.v20191230.SearchFaceResponse;
import com.ycwl.basic.config.FaceDetectConfig;
import com.ycwl.basic.constant.FaceConstant;
import com.ycwl.basic.exception.BaseException;
import com.ycwl.basic.mapper.FaceDetectLogMapper;
import com.ycwl.basic.mapper.FaceMapper;
import com.ycwl.basic.mapper.FaceSampleMapper;
import com.ycwl.basic.mapper.ScenicMapper;
import com.ycwl.basic.model.pc.face.entity.FaceEntity;
import com.ycwl.basic.model.pc.face.resp.FaceRespVO;
import com.ycwl.basic.model.pc.faceDetectLog.entity.FaceDetectLog;
import com.ycwl.basic.model.pc.faceDetectLog.resp.MatchLocalRecord;
import com.ycwl.basic.model.pc.faceSample.entity.FaceSampleEntity;
import com.ycwl.basic.model.pc.faceSample.req.FaceSampleReqQuery;
import com.ycwl.basic.model.pc.faceSample.resp.FaceSampleRespVO;
import com.ycwl.basic.model.pc.scenic.entity.ScenicConfigEntity;
import com.ycwl.basic.model.task.resp.AddFaceSampleRespVo;
import com.ycwl.basic.model.task.resp.SearchFaceRespVo;
import com.ycwl.basic.service.task.TaskFaceService;
import com.ycwl.basic.storage.StorageFactory;
import com.ycwl.basic.storage.adapters.IStorageAdapter;
import com.ycwl.basic.storage.utils.StorageUtil;
import com.ycwl.basic.utils.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.profile.DefaultProfile;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Slf4j
@Service
public class TaskFaceServiceImpl implements TaskFaceService {
    @Autowired
    private FaceMapper faceMapper;
    @Autowired
    private FaceSampleMapper faceSampleMapper;
    @Autowired
    private ScenicMapper scenicMapper;
    @Autowired
    private FaceDetectConfig faceDetectConfig;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private FaceDetectLogMapper logMapper;

    private IAcsClient getClient() {
        DefaultProfile profile = DefaultProfile.getProfile(
                faceDetectConfig.getRegion(),faceDetectConfig.getAccessKeyId(), faceDetectConfig.getAccessKeySecret());
        IAcsClient client = new DefaultAcsClient(profile);
        return client;
    }

    @Override
    public SearchFaceRespVo searchFace(Long faceId) {
        FaceRespVO faceRespVO = faceMapper.getById(faceId);
        SearchFaceRespVo respVo = searchFace(faceRespVO.getScenicId().toString(), faceRespVO.getFaceUrl());
        if (respVo != null) {
            FaceEntity faceEntity = new FaceEntity();
            faceEntity.setId(faceId);
            faceEntity.setMatchResult(respVo.getSearchResultJson());
            faceEntity.setFirstMatchRate(BigDecimal.valueOf(respVo.getFirstMatchRate()));
            faceEntity.setScore(respVo.getScore());
            faceEntity.setMatchSampleIds(StringUtils.join(respVo.getSampleListIds(), ","));
            faceMapper.update(faceEntity);
        }
        return respVo;
    }

    @Override
    public SearchFaceRespVo searchFace(Long scenicId, String faceUrl) {
        return searchFace(scenicId.toString(), faceUrl);
    }

    @Override
    public SearchFaceRespVo searchFace(String dbName, String faceUrl) {
        assureFaceDB(dbName);
        IAcsClient client = getClient();
        SearchFaceRequest request = new SearchFaceRequest();
        request.setDbName(dbName);
        request.setImageUrl(faceUrl);
        request.setLimit(100);
        request.setQualityScoreThreshold(80F);
        FaceDetectLog log = FaceDetectLog.quickCreate("预留字段", request);
        try {
            SearchFaceResponse response = client.getAcsResponse(request);
            log.fillResponse(response);
            List<SearchFaceResponse.Data.MatchListItem> matchList = response.getData().getMatchList();
            if (matchList.isEmpty()) {
                return null;
            }
            SearchFaceRespVo respVo = new SearchFaceRespVo();
            respVo.setScore(matchList.get(0).getQualitieScore());
            respVo.setSearchResultJson(JSON.toJSONString(matchList.get(0)));
            if (matchList.get(0).getFaceItems().isEmpty()) {
                return respVo;
            }
            List<MatchLocalRecord> records = matchList.get(0).getFaceItems().stream()
                    .map(item -> {
                        MatchLocalRecord record = new MatchLocalRecord();
                        record.setIdStr(item.getExtraData());
                        record.setFaceSampleId(Long.parseLong(item.getExtraData()));
                        if (StringUtils.isNumeric(item.getDbName())) {
                            record.setFaceUrl(getFaceSampleUrl(record.getFaceSampleId()));
                        } else {
                            record.setFaceUrl(getFaceUrl(record.getFaceSampleId()));
                        }
                        record.setConfidence(item.getConfidence());
                        return record;
                    })
                    .collect(Collectors.toList());
            log.matchLocalRecord(records);
            List<Long> faceSampleIds = records.stream()
                    .filter(record -> record.getConfidence() > 60)
                    .map(MatchLocalRecord::getFaceSampleId)
                    .collect(Collectors.toList());
            respVo.setFirstMatchRate(matchList.get(0).getFaceItems().get(0).getConfidence());
            respVo.setSampleListIds(faceSampleIds);
            return respVo;
        } catch (Exception e) {
            log.setMatchRawResult("识别错误，错误为：["+e.getLocalizedMessage()+"]");
            e.printStackTrace();
            throw new BaseException(e.getMessage());
        } finally {
            logMapper.insert(log);
        }
    }

    @Override
    public AddFaceSampleRespVo addFaceSample(Long faceSampleId) {
        FaceSampleRespVO faceSampleRespVO = faceSampleMapper.getById(faceSampleId);
        String entityId = generateEntityId(faceSampleRespVO);
        AddFaceSampleRespVo respVo = addFaceSample(faceSampleRespVO.getScenicId().toString(), entityId, faceSampleRespVO.getFaceUrl(), faceSampleId.toString());
        FaceSampleEntity faceSampleEntity = new FaceSampleEntity();
        faceSampleEntity.setId(faceSampleId);
        faceSampleEntity.setScore(respVo.getScore());
        faceSampleEntity.setUpdateAt(new Date());
        faceSampleMapper.update(faceSampleEntity);
        addFaceSampleUrlCache(faceSampleId, faceSampleRespVO.getFaceUrl());
        return respVo;
    }

    @Override
    public AddFaceSampleRespVo addFaceSample(String dbName, String entityId, String faceUrl, String extData) {
        AddFaceEntityRequest request = new AddFaceEntityRequest();
        request.setDbName(dbName);
        request.setEntityId(entityId);
        IAcsClient client = getClient();
        try {
            client.getAcsResponse(request);
        } catch (ClientException e) {
            return null;
        }
        AddFaceRequest addFaceRequest = new AddFaceRequest();
        addFaceRequest.setDbName(dbName);
        addFaceRequest.setEntityId(entityId);
        addFaceRequest.setImageUrl(faceUrl);
        addFaceRequest.setExtraData(extData);
        AddFaceSampleRespVo respVo = new AddFaceSampleRespVo();
        try {
            AddFaceResponse acsResponse = client.getAcsResponse(addFaceRequest);
            respVo.setScore(acsResponse.getData().getQualitieScore());
            return respVo;
        } catch (ClientException e) {
            return null;
        }
    }

    @Override
    public void batchDeleteExpiredFace(Long scenicId) {
        FaceSampleReqQuery query = new FaceSampleReqQuery();
        query.setDeviceId(scenicId);
        faceSampleMapper.list(query);
        ScenicConfigEntity scenicConfig = scenicMapper.getConfig(scenicId);
        if (scenicConfig == null) {
            return;
        }
        Integer sampleStoreDay = scenicConfig.getSampleStoreDay();
        if (sampleStoreDay == null) {
            sampleStoreDay = 3;
        }
        Date thatDay = DateUtil.offsetDay(new Date(), -sampleStoreDay);
        Date dayStart = DateUtil.beginOfDay(thatDay);
        Date dayEnd = DateUtil.endOfDay(thatDay);
        query.setStartTime(dayStart);
        query.setEndTime(dayEnd);
        IAcsClient client = getClient();
        faceSampleMapper.list(query).forEach(faceSampleEntity -> {
            String entityId = generateEntityId(faceSampleEntity);
            DeleteFaceEntityRequest request = new DeleteFaceEntityRequest();
            request.setDbName(scenicId.toString());
            request.setEntityId(entityId);
            try {
                client.getAcsResponse(request);
            } catch (ClientException e) {
                return;
            }
        });
    }

    @Override
    public void deleteFaceDB(String dbName) {
        ListFaceEntitiesRequest request = new ListFaceEntitiesRequest();
        request.setDbName(dbName);
        request.setLimit(200);
        IAcsClient client = getClient();
        try {
            while (true) {
                ListFaceEntitiesResponse response = client.getAcsResponse(request);
                if (response.getData().getTotalCount() == 0) {
                    break;
                }
                response.getData().getEntities().forEach(entity -> {
                    DeleteFaceEntityRequest deleteFaceEntityRequest = new DeleteFaceEntityRequest();
                    deleteFaceEntityRequest.setDbName(entity.getDbName());
                    deleteFaceEntityRequest.setEntityId(entity.getEntityId());
                    try {
                        Thread.sleep(1000);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                    try {
                        client.getAcsResponse(deleteFaceEntityRequest);
                    } catch (ClientException e) {
                        log.error("删除人脸数据失败！", e);
                    }
                });
            }
            DeleteFaceDbRequest deleteFaceDbRequest = new DeleteFaceDbRequest();
            deleteFaceDbRequest.setName(dbName);
            client.getAcsResponse(deleteFaceDbRequest);
            removeFaceDBCache(dbName);
        } catch (ClientException e) {
            log.error("删除人脸数据库失败！", e);
        }
    }

    @Override
    public void createFaceDB(String dbName) {
        try {
            CreateFaceDbRequest request = new CreateFaceDbRequest();
            request.setName(dbName);
            IAcsClient client = getClient();
            client.getAcsResponse(request);
            addFaceDBCache(dbName);
        } catch (ClientException e) {
            log.error("阿里云添加人脸数据库失败！", e);
        }
    }

    @Override
    public void assureFaceDB(String dbName) {
        if (redisTemplate.hasKey(FaceConstant.FACE_DB_NAME_PFX + dbName)) {
            return;
        }
        try {
            long offset = 0;
            List<ListFaceDbsResponse.Data.DbListItem> dbList = new ArrayList<>();
            IAcsClient client = getClient();
            while (true) {
                ListFaceDbsRequest request = new ListFaceDbsRequest();
                request.setLimit(200L);
                request.setOffset(offset);
                ListFaceDbsResponse response = client.getAcsResponse(request);
                List<ListFaceDbsResponse.Data.DbListItem> list = response.getData().getDbList();
                if (list == null || list.isEmpty()) {
                    break;
                } else {
                    dbList.addAll(list);
                    offset += list.size();
                }
            }
            boolean mismatch;
            if (dbList.isEmpty()) {
                clearFaceDBCache();
                mismatch = true;
            } else {
                mismatch = dbList.stream().peek(item -> {
                    redisTemplate.opsForValue().set(FaceConstant.FACE_DB_NAME_PFX + dbName, "1");
                }).noneMatch(db -> db.getName().equals(dbName));
            }
            if (mismatch) {
                createFaceDB(dbName);
            }
        } catch (ClientException e) {
            log.error("阿里云确保人脸数据库失败！", e);
        }
    }

    @Override
    public String uploadFile(MultipartFile file, Long userId) {
        if (file.isEmpty()) {
            throw new RuntimeException("文件不存在！");
        }
        String originalFilename = file.getOriginalFilename();
        //获取文件名后缀
        String suffix = originalFilename.split("\\.")[1];
        if ("Jpeg".equals(suffix)) {
            suffix = "jpg";
        }
        //文件储存路径
        String filePath = StorageUtil.joinPath("user-faces", DateUtils.format(new Date(),"yyyy-MM-dd"));
        // 生成文件名
        String fileName= userId + "." + suffix;
        IStorageAdapter adapter = StorageFactory.use("faces");
        try {
            return adapter.uploadFile(file.getInputStream(), filePath, fileName);
        } catch (IOException e) {
            log.error("文件上传失败！", e);
            return null;
        }
    }

    @Override
    public boolean deleteFaceSample(String dbName, String entityId) {
        DeleteFaceEntityRequest deleteFaceEntityRequest = new DeleteFaceEntityRequest();
        deleteFaceEntityRequest.setDbName(dbName);
        deleteFaceEntityRequest.setEntityId(entityId);
        try {
            IAcsClient client = getClient();
            client.getAcsResponse(deleteFaceEntityRequest);
            return true;
        } catch (ClientException e) {
            log.error("删除人脸样本失败！", e);
            return false;
        }
    }

    public void addFaceDBCache(String dbName) {
        redisTemplate.opsForValue().set(FaceConstant.FACE_DB_NAME_PFX + dbName, "1");
    }

    public Boolean removeFaceDBCache(String dbName) {
        return redisTemplate.delete(FaceConstant.FACE_DB_NAME_PFX + dbName);
    }

    public Boolean clearFaceDBCache() {
        return redisTemplate.delete(FaceConstant.FACE_DB_NAME_PFX + "*");
    }


    private static final String DATE_FORMAT="yyyyMMddHHmmssSSS";

    public static String generateEntityId(FaceSampleEntity entity) {
        SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT);
        String entityId = entity.getDeviceId().toString() + "_" + sdf.format(entity.getCreateAt());
        return entityId;
    }
    public static String generateEntityId(FaceSampleRespVO entity) {
        SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT);
        String entityId = entity.getDeviceId().toString() + "_" + sdf.format(entity.getCreateAt());
        return entityId;
    }

    public String getFaceUrl(Long faceId) {
        if (faceId == null) {
            return null;
        }
        if (redisTemplate.hasKey(FaceConstant.FACE_USER_URL_PFX + faceId)) {
            return redisTemplate.opsForValue().get(FaceConstant.FACE_USER_URL_PFX + faceId);
        }
        FaceRespVO faceRespVO = faceMapper.getById(faceId);
        if (faceRespVO == null) {
            return null;
        }
        String faceUrl = faceRespVO.getFaceUrl();
        if (StringUtils.isNotBlank(faceUrl)) {
            addFaceUrlCache(faceId, faceUrl);
        }
        return faceUrl;
    }
    public void addFaceUrlCache(Long faceId, String faceUrl) {
        if (faceId == null) {
            return;
        }
        if (StringUtils.isBlank(faceUrl)) {
            return;
        }
        redisTemplate.opsForValue().set(FaceConstant.FACE_USER_URL_PFX + faceId, faceUrl, 3, TimeUnit.DAYS);
    }
    public String getFaceSampleUrl(Long faceSampleId) {
        if (faceSampleId == null) {
            return null;
        }
        if (redisTemplate.hasKey(FaceConstant.FACE_SAMPLE_URL_PFX + faceSampleId)) {
            return redisTemplate.opsForValue().get(FaceConstant.FACE_SAMPLE_URL_PFX + faceSampleId);
        }
        FaceSampleRespVO faceSampleRespVO = faceSampleMapper.getById(faceSampleId);
        if (faceSampleRespVO == null) {
            return null;
        }
        String faceUrl = faceSampleRespVO.getFaceUrl();
        if (StringUtils.isNotBlank(faceUrl)) {
            addFaceSampleUrlCache(faceSampleId, faceUrl);
        }
        return faceUrl;
    }
    public void addFaceSampleUrlCache(Long faceSampleId, String faceUrl) {
        if (faceSampleId == null) {
            return;
        }
        if (StringUtils.isBlank(faceUrl)) {
            return;
        }
        redisTemplate.opsForValue().set(FaceConstant.FACE_SAMPLE_URL_PFX + faceSampleId, faceUrl, 3, TimeUnit.DAYS);
    }
}
