package com.ycwl.basic.service.impl.task;

import com.alibaba.fastjson.JSON;
import com.ycwl.basic.constant.TaskConstant;
import com.ycwl.basic.mapper.FaceMapper;
import com.ycwl.basic.mapper.FaceSampleMapper;
import com.ycwl.basic.mapper.RenderWorkerMapper;
import com.ycwl.basic.mapper.SourceMapper;
import com.ycwl.basic.mapper.TaskMapper;
import com.ycwl.basic.mapper.TemplateMapper;
import com.ycwl.basic.mapper.VideoMapper;
import com.ycwl.basic.model.pc.face.resp.FaceRespVO;
import com.ycwl.basic.model.pc.faceSample.resp.FaceSampleRespVO;
import com.ycwl.basic.model.pc.renderWorker.entity.RenderWorkerEntity;
import com.ycwl.basic.model.pc.source.entity.SourceEntity;
import com.ycwl.basic.model.pc.source.resp.SourceRespVO;
import com.ycwl.basic.model.pc.task.entity.TaskEntity;
import com.ycwl.basic.model.pc.task.req.TaskReqQuery;
import com.ycwl.basic.model.pc.task.resp.TaskRespVO;
import com.ycwl.basic.model.pc.template.entity.TemplateConfigEntity;
import com.ycwl.basic.model.pc.template.entity.TemplateEntity;
import com.ycwl.basic.model.pc.template.resp.TemplateRespVO;
import com.ycwl.basic.model.pc.video.entity.MemberVideoEntity;
import com.ycwl.basic.model.pc.video.entity.VideoEntity;
import com.ycwl.basic.model.task.req.ClientStatusReqVo;
import com.ycwl.basic.model.task.req.TaskReqVo;
import com.ycwl.basic.model.task.req.WorkerAuthReqVo;
import com.ycwl.basic.model.task.resp.TaskSyncRespVo;
import com.ycwl.basic.service.task.TaskService;
import com.ycwl.basic.storage.StorageFactory;
import com.ycwl.basic.storage.adapters.IStorageAdapter;
import com.ycwl.basic.task.VideoPieceGetter;
import com.ycwl.basic.utils.SnowFlakeUtil;
import lombok.NonNull;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Service
public class TaskTaskServiceImpl implements TaskService {
    @Autowired
    private TemplateMapper templateMapper;
    @Autowired
    private RenderWorkerMapper renderWorkerMapper;
    @Autowired
    private TaskMapper taskMapper;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private FaceMapper faceMapper;
    @Autowired
    private FaceSampleMapper faceSampleMapper;
    @Autowired
    private SourceMapper sourceMapper;
    @Autowired
    private VideoMapper videoMapper;

    private RenderWorkerEntity getWorker(@NonNull WorkerAuthReqVo req) {
        String accessKey = req.getAccessKey();
        if (accessKey == null) {
            return null;
        }
        RenderWorkerEntity worker = renderWorkerMapper.findByAccessKey(accessKey);
        return worker;
    }

    @Override
    public TaskSyncRespVo handleSyncTask(@NonNull TaskReqVo req) {
        RenderWorkerEntity worker = getWorker(req);
        if (worker == null) {
            return null;
        }
        worker.setOnline(1);
        worker.setName(null);
        worker.setStatus(null);
        // get status
        ClientStatusReqVo clientStatus = req.getClientStatus();
        if (clientStatus != null) {
            // 临时这么用下
            worker.setCpuCount(clientStatus.getCpu_count());
            worker.setCpuUsage(clientStatus.getCpu_usage());
            // 上报的是字节，存储的是兆
            worker.setMemoryAvailable(clientStatus.getMemory_available().divide(BigDecimal.valueOf(1024 * 1024), RoundingMode.CEILING));
            worker.setMemoryTotal(clientStatus.getMemory_total().divide(BigDecimal.valueOf(1024 * 1024), RoundingMode.CEILING));

            worker.setPlatform(clientStatus.getPlatform());
            worker.setRuntimeVersion(clientStatus.getRuntime_version());
            worker.setSupportFeature(String.join(",", clientStatus.getSupport_feature()));
            worker.setVersion(clientStatus.getVersion());

            worker.setUpdateAt(new Date());
            redisTemplate.opsForValue().set(TaskConstant.TASK_ONLINE_WORKER_KEY_PFX + worker.getId(), JSON.toJSONString(clientStatus), 60, TimeUnit.SECONDS);
        }
        renderWorkerMapper.update(worker);
        TaskSyncRespVo resp = new TaskSyncRespVo();
        // Template
//        String cacheJson = redisTemplate.opsForValue().get(TaskConstant.TASK_TEMPLATE_KEY_PFX + worker.getId());
//        if (cacheJson == null) {
//            TemplateReqQuery templateReqQuery = new TemplateReqQuery();
//            templateReqQuery.setPid(0L);
//            List<TemplateRespVO> templateList = templateMapper.list(templateReqQuery);
//            for (TemplateRespVO template : templateList) {
//                template.setChildren(templateMapper.getByPid(template.getId()));
//            }
//            resp.setTemplates(templateList);
//            redisTemplate.opsForValue().set(TaskConstant.TASK_TEMPLATE_KEY_PFX + worker.getId(), JSON.toJSONString(templateList), 60, TimeUnit.SECONDS);
//        } else {
//            resp.setTemplates(JSON.parseArray(cacheJson, TemplateRespVO.class));
//        }
        List<TaskRespVO> taskList = taskMapper.selectNotRunning();
        resp.setTasks(taskList);
        taskList.forEach(task -> taskMapper.assignToWorker(task.getId(), worker.getId()));
        // return Task
        return resp;
    }

    @Override
    public void createRenderTask(Long scenicId, Long templateId, Long faceId) {
        // 有人脸，找视频
        if (faceId == null) {
            return;
        }
        TemplateConfigEntity config = templateMapper.getConfig(templateId);
        FaceRespVO faceRespVO = faceMapper.getById(faceId);
        if (faceRespVO == null) {
            return;
        }
        Map<String, List<SourceRespVO>> sourcesMap = Arrays.stream(faceRespVO.getMatchSampleIds().split(","))
                .map(Long::valueOf)
                .map(sampleId -> faceSampleMapper.getById(sampleId))
                .filter(Objects::nonNull)
                .map(FaceSampleRespVO::getSourceId)
                .map(sourceId -> sourceMapper.getById(sourceId))
                .collect(Collectors.groupingBy(item -> item.getDeviceId().toString()));
        if (config != null) {
            if (config.getMinimalPlaceholderFill() > sourcesMap.size()) {
                throw new RuntimeException("请先游玩后在来生成吧~");
            }
        }
        TaskEntity taskEntity = new TaskEntity();
        taskEntity.setId(SnowFlakeUtil.getLongId());
        taskEntity.setFaceId(faceId);
        taskEntity.setMemberId(faceRespVO.getMemberId());
        taskEntity.setTemplateId(templateId);
        taskEntity.setScenicId(scenicId);
        taskEntity.setTaskParams(JSON.toJSONString(sourcesMap));
        taskEntity.setStatus(0);
        taskMapper.add(taskEntity);
    }

    @Override
    public TemplateRespVO workerGetTemplate(@NonNull Long templateId, @NonNull WorkerAuthReqVo req) {
        if (templateId == null) {
            return null;
        }
        RenderWorkerEntity worker = getWorker(req);
        if (worker == null) {
            return null;
        }
        TemplateRespVO respVO = templateMapper.getById(templateId);
        respVO.setChildren(templateMapper.getByPid(templateId));
        return respVO;
    }

    @Override
    public void taskStart(@NonNull Long taskId, @NonNull WorkerAuthReqVo req) {
        TaskRespVO task = taskMapper.getById(taskId);
        if (task == null) {
            return;
        }
        RenderWorkerEntity worker = getWorker(req);
        if (worker == null) {
            return;
        }
        TaskEntity taskUpdate = new TaskEntity();
        taskUpdate.setId(taskId);
        taskUpdate.setWorkerId(worker.getId());
        taskMapper.setStart(taskUpdate);
    }

    @Override
    public void autoCreateTaskByFaceId(Long faceId) {
        FaceRespVO faceRespVO = faceMapper.getById(faceId);
        if (faceRespVO == null) {
            return;
        }
        if (!StringUtils.isNotBlank(faceRespVO.getMatchSampleIds())) {
            return;
        }
        List<FaceSampleRespVO> faceSampleList = faceSampleMapper.listByIds(Arrays.stream(faceRespVO.getMatchSampleIds().split(",")).map(Long::valueOf).collect(Collectors.toList()));
        if (faceSampleList.isEmpty()) {
            return;
        }
        List<TemplateEntity> templateList = templateMapper.listEnabledByScenicId(faceRespVO.getScenicId());
        if (templateList == null || templateList.isEmpty()) {
            return;
        }
        TaskEntity faceAutomaticTask = taskMapper.getFaceAutomaticTask(faceId);
        if (faceAutomaticTask != null) {
            return;
        }
        createTaskByFaceIdAndTempalteId(faceId, templateList.get(0).getId(), 1);
    }
    @Override
    public void createTaskByFaceIdAndTempalteId(Long faceId, Long templateId) {
        createTaskByFaceIdAndTempalteId(faceId, templateId, 0);
    }

    @Override
    public void createTaskByFaceIdAndTempalteId(Long faceId, Long templateId, int automatic) {
        FaceRespVO faceRespVO = faceMapper.getById(faceId);
        List<FaceSampleRespVO> faceSampleList = faceSampleMapper.listByIds(Arrays.stream(faceRespVO.getMatchSampleIds().split(",")).map(Long::valueOf).collect(Collectors.toList()));
        if (faceSampleList.isEmpty()) {
            return;
        }
        List<Long> faceSampleIds = faceSampleList.stream().map(FaceSampleRespVO::getId).collect(Collectors.toList());
        List<SourceEntity> sourceList = sourceMapper.listBySampleIds(faceSampleIds);
        if (sourceList.isEmpty()) {
            return;
        }
        if (automatic > 0) {
            TaskReqQuery taskReqQuery = new TaskReqQuery();
            taskReqQuery.setFaceId(faceId);
            taskReqQuery.setTemplateId(templateId);
            List<TaskRespVO> list = taskMapper.list(taskReqQuery);
            if (!list.isEmpty()) {
                return;
            }
        }
        VideoPieceGetter.Task task = new VideoPieceGetter.Task();
        task.faceId = faceId;
        task.faceSampleId = faceSampleIds;
        task.memberId = faceRespVO.getMemberId();
        task.callback = () -> {
            List<SourceEntity> videoSourceList = sourceMapper.listVideoByFaceRelation(faceId);
            Map<String, List<SourceEntity>> sourcesMap = videoSourceList.stream()
                    .peek(item -> item.setUrl(item.getVideoUrl()))
                    .collect(Collectors.groupingBy(item -> item.getDeviceId().toString()));
            TaskReqQuery taskReqQuery = new TaskReqQuery();
            taskReqQuery.setFaceId(faceId);
            taskReqQuery.setTemplateId(templateId);
            taskReqQuery.setTaskParams(JSON.toJSONString(sourcesMap));
            List<TaskRespVO> list = taskMapper.list(taskReqQuery);

            MemberVideoEntity memberVideoEntity = new MemberVideoEntity();
            memberVideoEntity.setMemberId(faceRespVO.getMemberId());
            memberVideoEntity.setScenicId(faceRespVO.getScenicId());
            memberVideoEntity.setFaceId(faceId);
            memberVideoEntity.setTemplateId(templateId);
            memberVideoEntity.setIsBuy(0);
            if (list.isEmpty()) {
                TaskEntity taskEntity = new TaskEntity();
                taskEntity.setId(SnowFlakeUtil.getLongId());
                taskEntity.setScenicId(faceRespVO.getScenicId());
                taskEntity.setFaceId(faceId);
                taskEntity.setMemberId(faceRespVO.getMemberId());
                taskEntity.setTemplateId(templateId);
                taskEntity.setStatus(0);
                taskEntity.setAutomatic(automatic);
                taskEntity.setTaskParams(JSON.toJSONString(sourcesMap));
                taskMapper.add(taskEntity);
                memberVideoEntity.setTaskId(taskEntity.getId());
            } else{
                memberVideoEntity.setTaskId(list.get(0).getId());
                VideoEntity video = videoMapper.findByTaskId(list.get(0).getId());
                if (video != null) {
                    memberVideoEntity.setVideoId(video.getId());
                }
            }
            videoMapper.addRelation(memberVideoEntity);
        };
        VideoPieceGetter.addTask(task);
    }

    @Override
    public void taskSuccess(@NonNull Long taskId, @NonNull WorkerAuthReqVo req) {
        TaskRespVO task = taskMapper.getById(taskId);
        if (task == null) {
            return;
        }
        RenderWorkerEntity worker = getWorker(req);
        if (worker == null) {
            return;
        }
        TaskEntity taskUpdate = new TaskEntity();
        taskUpdate.setId(taskId);
        taskUpdate.setStatus(1);
        taskUpdate.setWorkerId(worker.getId());
        taskMapper.setSuccess(taskUpdate);
        VideoEntity video = videoMapper.findByTaskId(taskId);
        if (video != null) {
            video.setVideoUrl(task.getVideoUrl());
            videoMapper.update(video);
            videoMapper.updateRelationWhenTaskSuccess(taskId, video.getId());
        } else {
            video = new VideoEntity();
            video.setId(SnowFlakeUtil.getLongId());
            video.setScenicId(task.getScenicId());
            video.setMemberId(task.getMemberId());
            video.setTemplateId(task.getTemplateId());
            video.setTaskId(taskId);
            video.setWorkerId(worker.getId());
            video.setVideoUrl(task.getVideoUrl());
            video.setCreateTime(new Date());
            videoMapper.add(video);
            videoMapper.updateRelationWhenTaskSuccess(taskId, video.getId());
        }
    }

    @Override
    public void taskFail(Long taskId, WorkerAuthReqVo req) {
        TaskRespVO task = taskMapper.getById(taskId);
        if (task == null) {
            return;
        }
        RenderWorkerEntity worker = getWorker(req);
        if (worker == null) {
            return;
        }
        TaskEntity taskUpdate = new TaskEntity();
        taskUpdate.setId(taskId);
        taskUpdate.setStatus(2);
        taskUpdate.setWorkerId(worker.getId());
        taskMapper.setFail(taskUpdate);
        taskMapper.deassign(taskId);
    }

    @Override
    public String getUploadUrl(Long taskId, WorkerAuthReqVo req) {
        TaskRespVO task = taskMapper.getById(taskId);
        if (task == null) {
            return null;
        }
        IStorageAdapter adapter = StorageFactory.use("video");
        String filename = task.getId() + "_" + task.getScenicId() + ".mp4";
        if (StringUtils.isBlank(task.getVideoUrl())) {
            // 生成
            String url = adapter.getUrl(filename);
            TaskEntity updateTask = new TaskEntity();
            updateTask.setId(taskId);
            updateTask.setVideoUrl(url);
            taskMapper.update(updateTask);
        }
        return adapter.getUrlForUpload(filename);
    }
}
